#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

# Docs
module Puppet
  #
  Type.newtype(:wls_deployment) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-EOD
    This resource allows you to manage a cluster in an WebLogic domain.

    Here is an example on how you should use this:

        # 'jersey-bundle' will use default as wls_setting identifier
        wls_deployment{'jersey-bundle':
          ensure            => 'present',
          deploymenttype    => 'Library',
          stagingmode       => 'nostage',
          remote            => "1",
          upload            => "1",
          target            => ['AdminServer', 'WebCluster'],
          targettype        => ['Server', 'Cluster'],
          versionidentifier => '1.18@1.18.0.0',
        }

    In this example you are managing a deployment in the default domain. When you want to
    manage a deployment in a specific domain, you can use:

        wls_deployment{'my_domain/webapp':
          ensure            => 'present',
          deploymenttype    => 'AppDeployment',
          stagingmode       => 'nostage',
          remote            => "1",
          upload            => "1",
          target            => ['WebCluster'],
          targettype        => ['Cluster'],
          versionidentifier => '1.1@1.1.0.0',
          require           => Wls_deployment['jersey-bundle']
        }

    Check the [WebLogic documentation](https://docs.oracle.com/middleware/1212/wls/INTRO/deploying.htm#INTRO201) for more information
    about application deployment.

    EOD

    validate do
      return if self.ensure == :absent
      return if purging
      return if self.ensure == :absent
      fail "#{path}: localpath is a required parameter and cannot be empty." unless localpath
    end

    ensurable

    to_get_raw_resources do
      Puppet.info "index #{name}"
      environment = { 'action' => 'index', 'type' => 'wls_deployment' }
      wlst template('wls_config/wls_deployment/index.py.erb', binding), environment
    end

    on_create do
      Puppet.info "create #{name} "
      check_file_existence
      wlst template('wls_config/wls_deployment/create.py.erb', binding)
      nil
    end

    on_modify do
      Puppet.info "modify #{name} "
      check_file_existence
      wlst template('wls_config/wls_deployment/modify.py.erb', binding)
      nil
    end

    on_destroy do
      Puppet.info "destroy #{name} "
      wlst template('wls_config/wls_deployment/destroy.py.erb', binding)
      nil
    end

    parameter :name
    parameter :domain
    parameter :deployment_name
    parameter :localpath
    parameter :planpath
    parameter :timeout
    parameter :remote
    parameter :upload

    property :target
    property :targettype
    property :deploymenttype
    property :versionidentifier
    property :stagingmode
    property :deployment_order

    add_title_attributes(:deployment_name)

    private

    def check_file_existence
      fail "File #{localpath} specfied at parameter localpath doesn't exist." unless File.exist?(localpath)
      fail "File #{planpath} specfied at parameter planpath doesn't exist." if planpath && !File.exist?(planpath)
    end
  end
end
