#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  #
  Type.newtype(:wls_jms_connection_factory) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-EOD
    This resource allows you to manage a connection factory in a JMS Module of a WebLogic domain.

    Here is an example on how you should use this:

        wls_jms_connection_factory { 'jmsClusterModule:cf':
          ensure                    => 'present',
          attachjmsxuserid          => '0',
          clientidpolicy            => 'Restricted',
          defaulttargeting          => '0',
          jndiname                  => 'jms/cf',
          loadbalancingenabled      => '1',
          messagesmaximum           => '10',
          reconnectpolicy           => 'producer',
          serveraffinityenabled     => '1',
          subdeployment             => 'wlsServers',
          subscriptionsharingpolicy => 'Exclusive',
          transactiontimeout        => '3600',
          xaenabled                 => '0',
        }

    In this example you are managing a connection factory in the default domain. When you want to
    manage a connection factory in a specific domain, you can use:

        wls_jms_connection_factory { 'my_domain/jmsClusterModule:cf2':
          ensure                    => 'present',
          attachjmsxuserid          => '0',
          clientidpolicy            => 'Restricted',
          defaulttargeting          => '1',
          jndiname                  => 'jms/cf2',
          loadbalancingenabled      => '1',
          messagesmaximum           => '10',
          reconnectpolicy           => 'producer',
          serveraffinityenabled     => '1',
          subscriptionsharingpolicy => 'Exclusive',
          transactiontimeout        => '3600',
          xaenabled                 => '1',
        }

    EOD


    bean_path('/JMSSystemResources/%{jmsmodule}/JMSResource/%{jmsmodule}/ConnectionFactories')
    bean_type('ConnectionFactory')

    ensurable
    parameter :connection_factory_name
    parameter :domain
    parameter :name
    parameter :jmsmodule
    parameter :timeout

    property :jndiname
    property :localjndiname
    property :subdeployment
    property :defaulttargeting
    property :transactiontimeout
    property :xaenabled
    property :clientidpolicy
    property :subscriptionsharingpolicy
    property :messagesmaximum
    property :reconnectpolicy
    property :loadbalancingenabled
    property :serveraffinityenabled
    property :attachjmsxuserid
    property :defaultdeliverymode
    property :defaultredeliverydelay
    extraproperties do
      property :flow_control_enabled
      property :flow_interval
      property :flow_maximum
      property :flow_minimum
      property :flow_one_way_send_mode
      property :flow_one_way_send_window_size
      property :flow_steps
      property :transaction_timeout
      property :transaction_xa_connection_factory_enabled
      property :delivery_default_compression_threshold
      property :delivery_default_delivery_mode
      property :delivery_default_priority
      property :delivery_default_redelivery_delay
      property :delivery_default_time_to_deliver
      property :delivery_default_time_to_live
      property :delivery_default_unit_of_order
      property :delivery_send_timeout
      property :lb_load_balancing_enabled
      property :lb_server_affinity_enabled
      property :client_acknowledge_policy
      property :client_allow_close_in_on_message
      property :client_id_policy
      property :client_id
      property :client_messages_maximum
      property :client_multicast_overrun_policy
      property :client_reconnect_blocking_millis
      property :client_reconnect_policy
      property :client_subscription_sharing_policy
      property :client_synchronous_prefetch_mode
      property :client_total_reconnect_period_millis
      property :default_targeting_enabled
      property :jndi_name
      property :local_jndi_name
      property :notes
      property :security_attach_jmsx_user_id
      property :sub_deployment_name
      #
      # Weblogic 12.2.1 new properties
      #
    end

    add_title_attributes(:jmsmodule, :connection_factory_name) do
      /(.+):(.+)/
    end
  end
end
