#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_server_channel) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-EOD
    This resource allows you to manage a server channel in an WebLogic domain.

    Here is an example on how you should use this:

        wls_server_channel { 'wlsServer1:Channel-Cluster':
          ensure                      => 'present',
          channel_identity_customized => '0',
          client_certificate_enforced => '0',
          custom_identity_alias       => 'node1',
          enabled                     => '1',
          httpenabled                 => '1',
          listenaddress               => '10.10.10.100',
          listenport                  => '8003',
          max_message_size            => '25000000',
          outboundenabled             => '0',
          protocol                    => 'cluster-broadcast',
          publicaddress               => '10.10.10.100',
          publicport                  => '8003',
          tunnelingenabled            => '0',
          two_way_ssl                 => '0',
        }

    In this example you are managing a server channel in the default domain. When you want to
    manage a server channel in a specific domain, you can use:

        wls_server_channel { 'my_domain/wlsServer1:HTTP':
          ensure                      => 'present',
          channel_identity_customized => '0',
          client_certificate_enforced => '0',
          custom_identity_alias       => 'node1',
          enabled                     => '1',
          httpenabled                 => '1',
          listenport                  => '8004',
          max_message_size            => '35000000',
          outboundenabled             => '0',
          protocol                    => 'http',
          publicport                  => '8104',
          tunnelingenabled            => '0',
          two_way_ssl                 => '0',
        }

    Here are some more examples:

        wls_server_channel { 'default/wlsServer2:Channel-Cluster':
          ensure                      => 'present',
          channel_identity_customized => '0',
          client_certificate_enforced => '0',
          custom_identity_alias       => 'node2',
          enabled                     => '1',
          httpenabled                 => '1',
          listenaddress               => '10.10.10.200',
          listenport                  => '8003',
          max_message_size            => '25000000',
          outboundenabled             => '0',
          protocol                    => 'cluster-broadcast',
          publicaddress               => '10.10.10.200',
          publicport                  => '8003',
          tunnelingenabled            => '0',
          two_way_ssl                 => '0',
        }

        wls_server_channel { 'default/wlsServer2:HTTP':
          ensure                      => 'present',
          channel_identity_customized => '0',
          client_certificate_enforced => '0',
          custom_identity_alias       => 'node2',
          enabled                     => '1',
          httpenabled                 => '1',
          listenport                  => '8004',
          max_message_size            => '35000000',
          outboundenabled             => '0',
          protocol                    => 'http',
          publicport                  => '8104',
          tunnelingenabled            => '0',
          two_way_ssl                 => '0',
        }

    EOD

    bean_type('NetworkAccessPoint')
    bean_path('/Servers/%{server}/NetworkAccessPoints')

    ensurable
    parameter :domain
    parameter :name
    parameter :channel_name
    parameter :server
    parameter :timeout
    parameter :custom_identity_privatekey_passphrase

    property :protocol
    property :enabled
    property :listenport
    property :publicport
    property :listenaddress
    property :publicaddress
    property :httpenabled
    property :outboundenabled
    property :tunnelingenabled
    property :max_message_size
    property :custom_identity_alias
    property :two_way_ssl
    property :client_certificate_enforced
    property :channel_identity_customized
    extraproperties do
      property :login_timeout_millis
      property :outbound_enabled
      property :outbound_private_key_enabled
      property :accept_backlog
      property :channel_weight
      property :ciphersuites
      property :cluster_address
      property :custom_identity_key_store_type
      property :custom_private_key_alias
      property :custom_private_key_pass_phrase
      property :hostname_verification_ignored
      property :hostname_verifier
      property :max_backoff_between_failures
      property :notes
      property :outbound_private_key_alias
      property :outbound_private_key_pass_phrase
      property :private_key_pass_phrase
      property :proxy_address
      property :public_address
      property :public_port
      property :sdp_enabled
      property :tunneling_client_ping_secs
      property :tunneling_client_timeout_secs
      property :two_way_ssl_enabled
      property :use_fast_serialization
      property :allow_unencrypted_null_cipher
      property :complete_message_timeout
      property :connect_timeout
      property :custom_identity_key_store_file_name
      property :inbound_certificate_validation
      property :listen_address
      property :max_connected_clients
      property :outbound_certificate_validation
      property :private_key_alias
      property :custom_identity_key_store_pass_phrase
      property :http_enabled_for_this_protocol
      property :idle_connection_timeout
      property :listen_port
      property :proxy_port
      property :resolve_dns_name
      property :timeout_connection_with_pending_responses
      property :tunneling_enabled
      #
      # Weblogic 12.2.1 new properties
      #
    end

    add_title_attributes(:server, :channel_name) do
      /(.+):(.+)/
    end
  end
end
