#
# See the file "LICENSE" for the full license governing this code.
#
require 'tempfile'
require 'fileutils'
require 'puppet_x/enterprisemodules/weblogic/settings'
require 'puppet_x/enterprisemodules/weblogic/daemon'

module Puppet_X
  module EnterpriseModules
    module WebLogic
      module Access
        include Settings

        DEFAULT_FILE = '/etc/wls_setting.yaml'

        def self.included(parent)
          parent.extend(Access)
        end

        def puppet_version_higher_or_equal_then(version)
          current_version = Gem::Version.new(Puppet.version)
          requested_version = Gem::Version.new(version)
          current_version >= requested_version
        end
        #
        # Based on the value of ensure, do an autorequire or an autobefore
        #
        #
        def child_of(type, variable = nil, &proc)
          autorequire(type) do
            present = instance_eval('self[:ensure]') == :present
            if variable.nil? # Always
              instance_eval(&proc) if present
            else
              self[variable].nil? || self[variable].to_s == 'absent' ? nil : instance_eval(&proc) if present
            end
          end
          if puppet_version_higher_or_equal_then('4.0.0')
            #
            # Older puppet versions don't support autobefore.
            # Because we want to be compatible with puppet 3, we will Skip
            # this functionality on older versions.
            #
            autobefore(type) do
              absent = instance_eval('self[:ensure]') == :absent
              if absent
                if variable.nil? # Always
                  instance_eval(&proc)
                else
                  #
                  # Check the value of the provider is set
                  #
                  provider ? provider.send(variable) == :absent ? nil : full_resource(provider).instance_eval(&proc) : nil
                end
              end
            end
          else
            Puppet.debug "Skipping setting autobefore on #{type}"
          end
        end

        def wlst(content, parameters = {})
          script = 'wlstScript'
          action = ''
          unless parameters.nil?
            action = parameters['action']
            Puppet.debug "Executing: #{script} with action #{action}"
          else
            Puppet.debug "Executing: #{script} for a create,modify or destroy"
          end

          tmpFile = Tempfile.new([script, '.py'])
          tmpFile.write(content)
          tmpFile.close
          FileUtils.chmod(0555, tmpFile.path)

          csv_string = ''
          domains = configuration

          case action
          when 'index' then
            # if index do all domains
            i = 1
            domains.each do |key, values|
              Puppet.debug "domain found #{key}"
              csv_domain_string = execute_wlst(script, tmpFile, parameters, key, values, :return_output => true)
              if i > 1
                # with multi domain, remove first line if it is a header
                csv_domain_string = csv_domain_string.lines.to_a[1..-1].join
              end
              csv_string += csv_domain_string
              i += 1
            end
            convert_csv_data_to_hash(csv_string, [], :col_sep => ';')
          when 'execute' then
            domains.each do |key, values|
              # check content if we do this for the right domain
              if content.include? "real_domain='" + key
                csv_string = execute_wlst(script, tmpFile, parameters, key, values, :return_output => true)
              else
                Puppet.debug "Skip WLST for domain #{key}"
              end
            end
            convert_csv_data_to_hash(csv_string, [], :col_sep => ';')
          else
            #  Puppet.info "domain found #{domain}"
            domains.each do |key, values|
              # check content if we do this for the right domain
              if content.include? "real_domain='" + key
                Puppet.debug "Got the right domain #{key} script, now execute WLST"
                execute_wlst(script, tmpFile, parameters, key, values, :return_output => false)
              else
                Puppet.debug "Skip WLST for domain #{key}"
              end
            end
          end
        end

        private

        def config_file
          Pathname.new(DEFAULT_FILE).expand_path
        end

        def execute_wlst(script, tmpFile, parameters, domain, domainValues, options = {})
          operatingSystemUser       = domainValues['user']              || 'oracle'
          weblogicHomeDir           = domainValues['weblogic_home_dir']
          weblogicUser              = domainValues['weblogic_user']     || 'weblogic'
          weblogicConnectUrl        = domainValues['connect_url']       || 't3://localhost:7001'
          weblogicPassword          = domainValues['weblogic_password']
          postClasspath             = domainValues['post_classpath']
          custom_trust              = domainValues['custom_trust']
          trust_keystore_file       = domainValues['trust_keystore_file']
          trust_keystore_passphrase = domainValues['trust_keystore_passphrase']
          debug_module              = domainValues['debug_module']
          archive_path              = domainValues['archive_path']
          return_output              = options.fetch(:return_output) { false }

          fail('weblogic_home_dir cannot be nil, check the wls_setting resource type') if weblogicHomeDir.nil?
          fail('weblogic_password cannot be nil, check the wls_setting resource type') if weblogicPassword.nil?

          debugmode = Puppet::Util::Log.level
          if ENV['WLS_CONFIG_INFO'] == 'debug'
            puts 'Prepare to run: ' + tmpFile.path + ',' +  operatingSystemUser + ',' +  domain + ',' +  weblogicHomeDir + ',' +  weblogicUser + ',' +  weblogicPassword + ',' +  weblogicConnectUrl
            puts 'vvv==================================================================='
            File.open(tmpFile.path).readlines.each do |line|
              puts line
            end
            puts '^^^===================================================================='
          end

          wls_daemon = Daemon.run(operatingSystemUser, domain, weblogicHomeDir, weblogicUser, weblogicPassword, weblogicConnectUrl, postClasspath, custom_trust, trust_keystore_file, trust_keystore_passphrase)

          if debug_module.to_s == 'true'
            if !File.directory?(archive_path)
              FileUtils.mkdir(archive_path)
            end
            FileUtils.cp(tmpFile.path, archive_path)
          end

          if timeout_specified
            wls_daemon.execute_script(tmpFile.path, timeout_specified)
          else
            wls_daemon.execute_script(tmpFile.path)
          end
          File.read('/tmp/' + script + '.out') if return_output
        end

        #
        # Merges the information from the provider and the type into a usable reference
        # This is used when autoprging information
        #
        def full_resource(provider)
          property_hash = provider.instance_variable_get(:@property_hash)
          values = self.class.parameters.each do | parameter|
            property_hash[parameter] = self[parameter]
          end
          provider
        end

        def timeout_specified
          if respond_to?(:to_hash)
            to_hash.fetch(:timeout) { nil } #
          else
            nil
          end
        end
      end
    end
  end
end
