#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/weblogic/access'
require 'easy_type/template'
require 'easy_type/helpers'

module Puppet_X
  module EnterpriseModules
    module WebLogic
      class Tree

        def initialize(domain, provider = WlstProvider.new, logger = Puppet)
          @provider = provider
          @domain   = domain
          @provider.set_domain(domain)
          @provider_name = @provider.class.name
          @logger   = logger
        end


        ##
        #
        # @abstract Check if a path or a value exists.
        #
        # @param [String] the full path to get the value from
        #
        def exists?(path)
          on_error("Checking if path #{path} exists ") do
            @provider.exists?(path)
          end
        end

        ##
        #
        # @abstract Check if we need a restart
        #
        # @returns boolean
        #
        def needs_restart?
          on_error("checking if we need a restart") do
            @provider.needs_restart?
          end
        end

        ##
        #
        # @abstract Get the realm name and realm path value from WLST.
        #
        # @returns realm_name, realm_path
        #
        def realm
          on_error("fetching realm_name and realm_path") do
            @provider.realm
          end
        end

        ##
        #
        # @abstract Get a value from WLST.
        #
        # @param [String] the full path to get the value from
        #
        def get(path, &default_proc)
          on_error("fetching value from #{path}", default_proc) do
            @provider.get(path)
          end
        end

        ##
        #
        # @abstract Get the default value for the path
        #
        # @param [String] the full path to get the value from
        #
        def default_value(path, &default_proc)
          on_error("fetching default value for #{path}", default_proc) do
            @provider.default_value(path)
          end
        end

        ##
        #
        # @abstract Get a security policy for a resource from WLST.
        #
        # @param [String] resource type
        # @param [String] resource name
        #
        def get_security_policy(resource_type, name, &default_proc)
          on_error("fetching security policy for #{resource_type} #{name}", default_proc) do
            @provider.get_security_policy(resource_type, name)
          end
        end

        ##
        #
        # @abstract Set a security policy for a resource through WLST.
        #
        # @param [String] resource type
        # @param [String] resource name
        # @param [String] policy
        #
        def set_security_policy(resource_type, name, policy, &default_proc)
          on_error("Setting security policy for #{resource_type} #{name} to #{policy}", default_proc) do
            @provider.set_security_policy(resource_type, name, policy)
          end
        end


        ##
        #
        # @abstract Get an encrypted value from WLST.
        #
        # @param [String] the full path to get the value from
        #
        def get_encrypted(path, &default_proc)
          on_error("fetching value from #{path}", default_proc) do
            @provider.get_encrypted(path)
          end
        end

        ##
        #
        # @abstract Fetch all directories from a specified path
        #
        def children(path, &default_proc)
          on_error("fetching children from #{path}", default_proc) do
            @provider.children(path)
          end
        end

        ##
        #
        # @abstract Fetch all properties from a specified path
        #
        def properties(path, &default_proc)
          on_error("fetching properties from #{path}", default_proc) do
            @provider.properties(path)
          end
        end

        ##
        #
        # @abstract Create a bean with a specified name and type
        #
        #
        def create(bean_type, path, name)
          on_error("creating bean #{bean_type} in path #{path } with name #{name}") do
            @provider.create(bean_type, path, name)
          end
        end

        ##
        #
        # @abstract Create a property with a specified path
        #
        #
        def create_property(path)
          on_error("creating property at path #{path}") do
            @provider.create_property(path)
          end
        end

        ##
        #
        # @abstract Create a bean with a specified name and type
        #
        #
        def destroy(bean_type, path, name )
          on_error("destroying bean #{name} at #{path} of type #{bean_type}") do
            @provider.destroy(bean_type, path, name)
          end
        end

        def save(message = 'Saving current edits')
          on_error(message) do
            @provider.save
          end
        end

        def edit()
          on_error("Internal error on start edit session") do
            @provider.edit
          end
        end

        def clear(path)
          on_error("clearing #{path}") do
            @provider.clear(path)
          end
        end

        def set(path, value)
          on_error("setting #{path} to #{value}") do
            @provider.set(path, value)
          end
        end

        def set_reference(path, parent, value)
          on_error("setting #{path} to reference of #{value} in #{parent}") do
            @provider.set_reference(path, parent, value)
          end
        end

        def set_array(path, value)
          on_error("setting #{path} to array with values #{value}") do
            @provider.set_array(path, value)
          end
        end

        def set_references(path, parent, value)
          on_error("setting #{path} to array of referenced with values #{value}") do
            @provider.set_references(path, parent, value)
          end
        end

        def set_property(path, value)
          on_error("setting #{path} to property with values #{value}") do
            raise ArgumentError, 'absent value not suppported on properties' if value.to_s == 'absent'
            @provider.set_property(path, value)
          end
        end

        def set_property_tree(path, value, bean_type='Property')
          on_error("setting #{path} to property tree with values #{value}") do
            raise ArgumentError, 'absent value not suppported on property trees' if value.to_s == 'absent'
            @provider.set_property_tree(path, value, bean_type)
          end
        end

        def set_target(path, target, type)
          on_error("setting target on path #{path} to #{target} with type #{type}") do
            raise ArgumentError, 'absent value not suppported on targets' if target == 'absent' || target.include?('absent')
            @provider.set_target(path, target, type)
          end
        end

        def execute(script)
          on_error("executing script #{script}") do
            @provider.execute(script)
          end
        end

        private

          def on_error(message, default_proc = nil)
            begin
              @logger.debug message
              yield
            rescue RuntimeError => error
              if default_proc
                default_proc.call
              else
                fail "Error when #{message} in domain #{@domain}, using provider #{@provider_name}.\n#{error}"
              end
            end
          end

      end

      class WlstProvider
        include Puppet_X::EnterpriseModules::WebLogic::Access
        include EasyType::Template
        include EasyType::Helpers

        def set_domain(domain)
          @domain = domain
        end

        def get(path)
          value = wlst template("wls_config/wls_config/tree/get.py.erb", binding), query_environment
          value.first['value']
        end

        def needs_restart?
          value = wlst template("wls_config/wls_config/tree/needs_restart.py.erb", binding), query_environment
          value.first['value']
        end

        def get_security_policy(resource_type, name)
          value = wlst template("wls_config/wls_config/tree/get_security_policy.py.erb", binding), query_environment
          value.first['value']
        end

        def set_security_policy(resource_type, name, policy)
          value = wlst template("wls_config/wls_config/tree/set_security_policy.py.erb", binding), query_environment
        end

        def default_value(path)
          value = wlst template("wls_config/wls_config/tree/default_value.py.erb", binding), query_environment
          value.first['value']
        end

        def realm()
          value = wlst template("wls_config/wls_config/tree/realm.py.erb", binding), query_environment
          if value.empty?
            [nil,nil]
          else
            [value.first['name'], value.first['path']]
          end
        end


        def exists?(path)
          value = wlst template("wls_config/wls_config/tree/exists.py.erb", binding), query_environment
          case value.first['value']
          when '1' then true
          when '0' then false
          else fail 'invalid value returned from exists? function'
          end
        end

        def get_encrypted(path)
          value = wlst template("wls_config/wls_config/tree/get_encrypted.py.erb", binding), query_environment
          value.first['value']
        end

        def create(bean_type, path, name)
          wlst template("wls_config/wls_config/tree/create.py.erb", binding), modify_environment
        end

        def create_property(path)
          wlst template("wls_config/wls_config/tree/create_property.py.erb", binding), modify_environment
        end

        def destroy(bean_type, path, name)
          wlst template("wls_config/wls_config/tree/destroy.py.erb", binding), modify_environment
        end

        def clear(path)
          wlst template("wls_config/wls_config/tree/clear.py.erb", binding), modify_environment
        end

        def set(path, value)
          wlst template("wls_config/wls_config/tree/set.py.erb", binding), modify_environment
        end

        def set_reference(path, parent, value)
          wlst template("wls_config/wls_config/tree/set_reference.py.erb", binding), modify_environment
        end

        def set_array(path, value)
          wlst template("wls_config/wls_config/tree/set_array.py.erb", binding), modify_environment
        end

        def set_references(path, parent, value)
          wlst template("wls_config/wls_config/tree/set_references.py.erb", binding), modify_environment
        end

        def set_property(path, value)
          wlst template("wls_config/wls_config/tree/set_property.py.erb", binding), modify_environment
        end

        def set_property_tree(path, value, bean_type)
          wlst template("wls_config/wls_config/tree/set_property_tree.py.erb", binding), modify_environment
        end

        def set_target(path, target, type)
          wlst template("wls_config/wls_config/tree/set_target.py.erb", binding), modify_environment
        end

        def edit
          wlst template("wls_config/wls_config/tree/edit.py.erb", binding), modify_environment
        end

        def save
          wlst template("wls_config/wls_config/tree/save.py.erb", binding), modify_environment
        end

        def execute(script)
          wlst script, modify_environment
        end

        def children(path)
          elements(path,'c') # Fetch only children
        end

        def properties(path)
          elements(path,'a') # Fetch only attributes
        end

        private

        def elements(path, type)
          value = wlst template("wls_config/wls_config/tree/directory.py.erb", binding), query_environment
          if value.empty?
            []
          else
            elements = value.first['value']
            elements ? elements.split(',') : []
          end
        end

        def query_environment
          { 'action' => 'execute', 'type' => :wls_tree }
        end

        def modify_environment
          { 'action' => 'modify', 'type' => :wls_tree }
        end

      end
    end
  end
end
